<?php
class WPJAM_Content_Template_Builtin{
	public static function get_fields($id='', $action_key=0){
		if($action_key == 'add'){
			return ['template_type'	=> ['type'=>'radio',	'sep'=>'<br /><br />',	'options'=>WPJAM_Content_Template_Type::get_options()]];
		}
	}

	public static function list_action($id, $data, $action_key=''){
		if($action_key == 'add'){
			$template_type	= $data['template_type'] ?: 'content';

			if($template_type == 'content'){
				return admin_url('post-new.php?post_type=template&template_type=content');
			}else{
				return admin_url('edit.php?post_type=template&page=wpjam-'.$template_type);
			}
		}
	}

	public static function on_pre_get_posts($query){
		if($query->is_main_query()){
			$type	= wpjam_get_data_parameter('template_type',	['sanitize_callback'=>'sanitize_key']);

			if($type){
				$query->set('meta_key', '_template_type');

				if($type == 'content'){
					$query->set('meta_compare', 'NOT EXISTS');
				}else{
					$query->set('meta_value', $type);
				}
			}
		}
	}

	public static function filter_views($views){
		$current	= wpjam_get_data_parameter('template_type');

		foreach(WPJAM_Content_Template_Type::get_registereds() as $type => $object) {
			$query_args	= $type == 'content' ? ['meta_compare'=>'NOT EXISTS'] : ['meta_value'=>$type];
			$query_args	= array_merge(['no_found_rows'=>false, 'post_type'=>'template',	'meta_key'=>'_template_type'], $query_args);
			$query		= wpjam_query($query_args);

			if($count = $query->found_posts){
				$class	= ($current && $current == $type) ?' class="current"' : '';
				$views[$type.'-content']	='<a href="edit.php?post_type=template&template_type='.$type.'"'.$class.'><span class="dashicons dashicons-'.$object->dashicon.'"></span> '.$object->title.'模板<span class="count">（'.$count.'）</span></a>';
			}
		}

		return $views;
	}

	public static function filter_single_row($single_row, $post_id){
		$type	= wpjam_get_content_template_type($post_id);

		if($type){
			return str_replace(
				'href="'.admin_url('post.php?post='.$post_id.'&amp;action=edit').'"',
				'href="'.admin_url('edit.php?post_type=template&amp;page=wpjam-'.$type.'&post_id='.$post_id).'"',
				$single_row
			);
		}

		return $single_row;
	}

	public static function column_callback($post_id, $column_name){
		if($column_name == 'shortcode'){
			$shortcode	= '[template id="'.$post_id.'"]';
			$post_name	= get_post($post_id)->post_name;

			if($post_name && $post_name != $post_id && strpos($post_name, '%') === false){
				$shortcode	.= "\n\n".'[template name="'.$post_name.'"]';
			}

			return wpautop($shortcode);
		}elseif($column_name == 'template_type'){
			$object	= wpjam_get_content_template_type_object($post_id);

			return $object ? '<span class="dashicons dashicons-'.$object->dashicon.'"></span>  '.$object->title.'模板' : '';
		}
	}

	public static function show_if($post_id, $action_key){
		if(get_post_status($post_id) == 'publish'){
			$object	= wpjam_get_content_template_type_object($post_id);

			return $object ? method_exists($object->model, 'export') : false;
		}

		return false;
	}

	public static function add_editor_button($editor_id){
		echo ' '.wpjam_get_page_button('insert_content_template', ['data'=>['editor_id'=>$editor_id]]);
		?>
		<script type="text/javascript">
		jQuery(function($){
			$('body').on('submit', "#template_form", function(e){
				e.preventDefault();	// 阻止事件默认行为。
				wp.media.editor.insert("\n"+'[template id="'+$('#template_id').val()+'"]'+"\n");
				tb_remove();
			});
		});
		</script>

		<?php
	}

	public static function page_load($screen){
		if($screen->post_type == 'template'){
			if($screen->base == 'edit'){
				wpjam_register_list_table_column('template_type',	[
					'title'		=> '类型',
					'callback'	=> [self::class, 'column_callback']
				]);

				wpjam_register_list_table_column('shortcode',		[
					'title'		=> '短代码',
					'callback'	=> [self::class, 'column_callback']
				]);

				wpjam_register_list_table_action('add', [
					'title'		=> '新建',
					'response'	=> 'redirect',
					'width'		=> 400,
					'fields'	=> [self::class, 'get_fields'],
					'callback'	=> [self::class, 'list_action']
				]);

				wpjam_register_list_table_action('export',	[
					'title'		=> '导出',
					'show_if'	=> [self::class, 'show_if'],
					'redirect'	=> admin_url('edit.php?post_type=template&export_action&id=%id%')
				]);

				wpjam_unregister_list_table_action('set_post_type');
				wpjam_unregister_list_table_action('upload_external_images');

				add_filter('disable_months_dropdown', '__return_true');

				add_filter('views_edit-template',	[self::class, 'filter_views'], 1, 2);
				add_filter('wpjam_single_row',		[self::class, 'filter_single_row'], 10, 2);

				add_action('pre_get_posts',			[self::class, 'on_pre_get_posts']);

				if(isset($_REQUEST['export_action'])){
					if(!current_user_can('manage_options')){
						return wp_die('无权限！');
					}

					$post_id	= (int)wpjam_get_data_parameter('id');
					$object		= wpjam_get_content_template_type_object($post_id);


					if(!$object || !method_exists($object->model, 'export')){
						wp_die('无效的模板');
					}

					$object->export($post_id);
				}
			}else{
				remove_meta_box('slugdiv', 'template', null);

				$post_id	= $_GET['post'] ?? 0;
				$_post		= $post_id ? get_post($post_id) : null;
				$post_name	= $_post ? $_post->post_name : '';

				wpjam_register_post_option('name_meta_box', [
					'title'		=> '标识',
					'context'	=> 'side',
					'callback'	=> '__return_null',
					'fields'	=> ['post_name'=>['title'=>'', 'value'=>$post_name, 'type'=>'text', 'class'=>'', 'style'=>'width:98%;']]
				]);
			}
		}elseif($screen->post_type != 'attachment' && $screen->base == 'post') {
			if(current_user_can('edit_posts')){
				wpjam_register_page_action('insert_content_template', [
					'page_title'	=> '插入模板',
					'form_id'		=> 'template_form',
					'submit_text'	=> '插入',
					'button_text'	=> '<span class="dashicons dashicons-edit content-template-button" style="width:18px; height:18px; vertical-align:text-bottom;"></span> 插入模板', 
					'class'			=> 'button',
					'width'			=> 420,
					'fields'		=> ['template'	=> ['title'=>'',	'type'=>'fieldset',	'fields'=>[
						'template_id'	=> array_merge(wpjam_get_post_id_field('template'), ['title'=>'选择模板']),
						'template_view'	=> ['title'=>' ',	'type'=>'view',	'value'=>'请点击选择或者输入关键字查询后选择...'],
					]]],
				]);

				add_action('media_buttons',	[self::class, 'add_editor_button']);
			}
		}
	}
}